#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Trailing Resistance &nd Support Stops (Tr&nds) as discussed in Sylvain Vervoort's July 2009 S&C article.
    /// </summary>
    [Description("Trailing Resistance and Support Stops (Trends) as discussed in Sylvain Vervoort's July 2009 S&C article.")]
    public class TrendsTrailingStop : Indicator
    {
        #region Variables
		private int		period = 10; // Default setting for Period
		private double	multi = 2.8; // Default setting for Multiplication
		
		private DataSeries highLow;
		private DataSeries diff;
		
		private double	hiLo		= 0;
		private double	hRef		= 0;
		private double	lRef		= 0;
		private double	resistance	= 0;
		private double	support		= 0;
		private double	trends		= 0;
		private bool	reverse		= false;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.Blue, PlotStyle.Line, "Tr&nds"));
            Overlay				= true;
            PriceTypeSupported	= false;
			
			highLow = new DataSeries(this);
			diff	= new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			hiLo = High[0] - Low[0];
			highLow.Set(hiLo);
			if (CurrentBar < 4)
				return;
			
			if (hiLo >= (1.5 * SMA(highLow, Period)[0]))
				hiLo = 1.5 * SMA(highLow, Period)[0];
			
			hRef = Low[0] <= High[1] ? High[0] - Close[1] : (High[0] - Close[1]) - ((Low[0] - High[1]) / 2);
			lRef = High[0] >= Low[1] ? Close[1] - Low[0] : (Close[1] - Low[0]) - ((Low[1] - High[0]) / 2);

			diff.Set(Math.Max(Math.Max(hiLo, hRef), lRef));
			resistance = Close[0] + Multi * WWMA(diff, Period)[0];
			
			if (Low[0] >= Low[2] && Low[1] >= Low[2] && Low[3] >= Low[2] && Low[4] >= Low[2])
				support = Low[2];
			else if (Low[0] > High[1] * 1.0013)
				support = High[1] * 0.9945;
			else if (Low[0] > Value[1] * 1.1)
				support = Value[1] * 1.05;
			else
				support = Value[1];
			
			if (High[0] > Value[1] && High[1] > Value[1])
			{
				trends = Math.Max(Value[1], support);
				if (reverse)
				{
					DrawArrowUp(CurrentBar.ToString(), false, 1, Value[1], Color.Orange);
					reverse = false;
				}
			}
			else if (High[0] < Value[1] && High[1] < Value[1])
			{
				trends = Math.Min(Value[1], resistance);
				if (reverse)
				{
					DrawArrowDown(CurrentBar.ToString(), false, 1, Value[1], Color.Orange);
					reverse = false;
				}
			}
			else if (High[0] >= Value[1])
			{
				trends = support;
				reverse = true;
			}
			else
			{
				trends = resistance;
				reverse = true;
			}
			
			Value.Set(trends);
        }

        #region Properties
		[Description("ATR Period")]
        [Category("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Min(Math.Max(1, value), 100); }
        }
		
        [Description("ATR multiplication")]
        [Category("Parameters")]
        public double Multi
        {
            get { return multi; }
            set { multi = Math.Min(Math.Max(1, value), 10); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TrendsTrailingStop[] cacheTrendsTrailingStop = null;

        private static TrendsTrailingStop checkTrendsTrailingStop = new TrendsTrailingStop();

        /// <summary>
        /// Trailing Resistance and Support Stops (Trends) as discussed in Sylvain Vervoort's July 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        public TrendsTrailingStop TrendsTrailingStop(double multi, int period)
        {
            return TrendsTrailingStop(Input, multi, period);
        }

        /// <summary>
        /// Trailing Resistance and Support Stops (Trends) as discussed in Sylvain Vervoort's July 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        public TrendsTrailingStop TrendsTrailingStop(Data.IDataSeries input, double multi, int period)
        {
            checkTrendsTrailingStop.Multi = multi;
            multi = checkTrendsTrailingStop.Multi;
            checkTrendsTrailingStop.Period = period;
            period = checkTrendsTrailingStop.Period;

            if (cacheTrendsTrailingStop != null)
                for (int idx = 0; idx < cacheTrendsTrailingStop.Length; idx++)
                    if (Math.Abs(cacheTrendsTrailingStop[idx].Multi - multi) <= double.Epsilon && cacheTrendsTrailingStop[idx].Period == period && cacheTrendsTrailingStop[idx].EqualsInput(input))
                        return cacheTrendsTrailingStop[idx];

            TrendsTrailingStop indicator = new TrendsTrailingStop();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Multi = multi;
            indicator.Period = period;
            indicator.SetUp();

            TrendsTrailingStop[] tmp = new TrendsTrailingStop[cacheTrendsTrailingStop == null ? 1 : cacheTrendsTrailingStop.Length + 1];
            if (cacheTrendsTrailingStop != null)
                cacheTrendsTrailingStop.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTrendsTrailingStop = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Trailing Resistance and Support Stops (Trends) as discussed in Sylvain Vervoort's July 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TrendsTrailingStop TrendsTrailingStop(double multi, int period)
        {
            return _indicator.TrendsTrailingStop(Input, multi, period);
        }

        /// <summary>
        /// Trailing Resistance and Support Stops (Trends) as discussed in Sylvain Vervoort's July 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        public Indicator.TrendsTrailingStop TrendsTrailingStop(Data.IDataSeries input, double multi, int period)
        {
            return _indicator.TrendsTrailingStop(input, multi, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Trailing Resistance and Support Stops (Trends) as discussed in Sylvain Vervoort's July 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TrendsTrailingStop TrendsTrailingStop(double multi, int period)
        {
            return _indicator.TrendsTrailingStop(Input, multi, period);
        }

        /// <summary>
        /// Trailing Resistance and Support Stops (Trends) as discussed in Sylvain Vervoort's July 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        public Indicator.TrendsTrailingStop TrendsTrailingStop(Data.IDataSeries input, double multi, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TrendsTrailingStop(input, multi, period);
        }

    }
}
#endregion
